@@ -37,6 +37,9 @@
 #include "tracker-collation.h"
 
 #include "tracker-db-interface-sqlite.h"
+#include "tracker-db-manager.h"
+
+#define UNKNOWN_STATUS 0.5
 
 typedef struct {
 	TrackerDBStatement *head;
@@ -55,7 +58,13 @@ struct TrackerDBInterface {
 
 	GSList *function_data;
 
+	/* Collation and locale change */
 	gpointer collator;
+	gpointer locale_notification_id;
+	gint collator_reset_requested;
+
+	/* Number of active cursors */
+	gint n_active_cursors;
 
 	guint ro : 1;
 #if HAVE_TRACKER_FTS
@@ -63,12 +72,14 @@ struct TrackerDBInterface {
 #endif
 	GCancellable *cancellable;
 
-	/* async operation pending */
-	gboolean pending;
 	GAsyncReadyCallback outstanding_callback;
 
 	TrackerDBStatementLru select_stmt_lru;
 	TrackerDBStatementLru update_stmt_lru;
+
+	TrackerBusyCallback busy_callback;
+	gpointer busy_user_data;
+	gchar *busy_status;
 };
 
 struct TrackerDBInterfaceClass {
@@ -84,6 +95,8 @@ struct TrackerDBCursor {
 	gint n_types;
 	gchar **variable_names;
 	gint n_variable_names;
+
+	gboolean threadsafe;
 };
 
 struct TrackerDBCursorClass {
@@ -111,7 +124,8 @@ static TrackerDBCursor    *tracker_db_cursor_sqlite_new      (sqlite3_stmt
                                                               TrackerPropertyType  *types,
                                                               gint                  n_types,
                                                               const gchar         **variable_names,
-                                                              gint                  n_variable_names);
+                                                              gint                  n_variable_names,
+                                                              gboolean              threadsafe);
 static gboolean            tracker_db_cursor_get_boolean     (TrackerSparqlCursor  *cursor,
                                                               guint                 column);
 static gboolean            db_cursor_iter_next               (TrackerDBCursor      *cursor,
@@ -510,9 +524,27 @@ static int
 check_interrupt (void *user_data)
 {
 	TrackerDBInterface *db_interface = user_data;
+
+	if (db_interface->busy_callback) {
+		db_interface->busy_callback (db_interface->busy_status,
+		                             UNKNOWN_STATUS, /* No idea to get the status from SQLite */
+		                             db_interface->busy_user_data);
+	}
+
 	return g_cancellable_is_cancelled (db_interface->cancellable) ? 1 : 0;
 }
 
+static void
+tracker_locale_notify_cb (TrackerLocaleID id,
+                          gpointer        user_data)
+{
+	TrackerDBInterface *db_interface = user_data;
+
+	/* Request a collator reset. Use thread-safe methods as this function will get
+	 * called from the main thread */
+	g_atomic_int_compare_and_exchange (&(db_interface->collator_reset_requested), FALSE, TRUE);
+}
+
 static void
 open_database (TrackerDBInterface *db_interface)
 {
@@ -533,18 +565,12 @@ open_database (TrackerDBInterface *db_interface)
 	}
 
 	/* Set our unicode collation function */
-	if (!db_interface->collator) {
-		db_interface->collator = tracker_collation_init ();
-		if (sqlite3_create_collation (db_interface->db,
-		                              TRACKER_COLLATION_NAME,
-		                              SQLITE_UTF8,
-		                              db_interface->collator,
-		                              tracker_collation_utf8) != SQLITE_OK)
-		{
-			g_critical ("Couldn't set collation function: %s",
-			            sqlite3_errmsg (db_interface->db));
-		}
-	}
+	tracker_db_interface_sqlite_reset_collator (db_interface);
+	/* And register for updates on locale changes */
+	db_interface->locale_notification_id = tracker_locale_notify_add (TRACKER_LOCALE_COLLATE,
+	                                                                  tracker_locale_notify_cb,
+	                                                                  db_interface,
+	                                                                  NULL);
 
 	sqlite3_progress_handler (db_interface->db, 100,
 	                          check_interrupt, db_interface);
@@ -590,8 +616,8 @@ tracker_db_interface_sqlite_constructor (GType                  type,
 	TrackerDBInterface *db_iface;
 
 	object = (* G_OBJECT_CLASS (tracker_db_interface_parent_class)->constructor) (type,
-		               n_construct_properties,
-		               construct_params);
+		        n_construct_properties,
+		        construct_params);
 	db_iface = TRACKER_DB_INTERFACE (object);
 
 	open_database (db_iface);
@@ -707,6 +733,28 @@ tracker_db_interface_sqlite_fts_update_rollback (TrackerDBInterface *db_interfac
 }
 #endif
 
+void
+tracker_db_interface_sqlite_reset_collator (TrackerDBInterface *db_interface)
+{
+	g_debug ("Resetting collator in db interface %p", db_interface);
+	if (db_interface->collator)
+		tracker_collation_shutdown (db_interface->collator);
+
+	db_interface->collator = tracker_collation_init ();
+	/* This will overwrite any other collation set before, if any */
+	if (sqlite3_create_collation (db_interface->db,
+	                              TRACKER_COLLATION_NAME,
+	                              SQLITE_UTF8,
+	                              db_interface->collator,
+	                              tracker_collation_utf8) != SQLITE_OK)
+	{
+		g_critical ("Couldn't set collation function: %s",
+		            sqlite3_errmsg (db_interface->db));
+	}
+}
+
+
+
 static void
 tracker_db_interface_sqlite_finalize (GObject *object)
 {
@@ -719,7 +767,9 @@ tracker_db_interface_sqlite_finalize (GObject *object)
 	g_message ("Closed sqlite3 database:'%s'", db_interface->filename);
 
 	g_free (db_interface->filename);
+	g_free (db_interface->busy_status);
 
+	tracker_locale_notify_remove (db_interface->locale_notification_id);
 	tracker_collation_shutdown (db_interface->collator);
 
 	G_OBJECT_CLASS (tracker_db_interface_parent_class)->finalize (object);
@@ -843,10 +893,29 @@ tracker_db_interface_set_max_stmt_cache_size (TrackerDBInterface         *db_int
 	}
 
 	/* Must be larger than 2 to make sense (to have a tail and head) */
-	if (max_size > 2)
+	if (max_size > 2) {
 		stmt_lru->max = max_size;
-	else
+	} else {
 		stmt_lru->max = 3;
+	}
+}
+
+void
+tracker_db_interface_set_busy_handler (TrackerDBInterface  *db_interface,
+                                       TrackerBusyCallback  busy_callback,
+                                       const gchar         *busy_status,
+                                       gpointer             busy_user_data)
+{
+	g_return_if_fail (TRACKER_IS_DB_INTERFACE (db_interface));
+	db_interface->busy_callback = busy_callback;
+	db_interface->busy_user_data = busy_user_data;
+	g_free (db_interface->busy_status);
+
+	if (busy_status) {
+		db_interface->busy_status = g_strdup (busy_status);
+	} else {
+		db_interface->busy_status = NULL;
+	}
 }
 
 TrackerDBStatement *
@@ -862,18 +931,17 @@ tracker_db_interface_create_statement (TrackerDBInterface           *db_interfac
 	gchar *full_query;
 
 	g_return_val_if_fail (TRACKER_IS_DB_INTERFACE (db_interface), NULL);
-	g_return_val_if_fail (!db_interface->pending, NULL);
 
 	va_start (args, query);
 	full_query = g_strdup_vprintf (query, args);
 	va_end (args);
 
-	/* There are three kinds of queries: 
-	  * a) Cached queries: SELECT and UPDATE ones (cache_type)
-	  * b) Non-Cached queries: NONE ones (cache_type)
-	  * c) Forced Non-Cached: in case of a stmt being already in use, we can't
-	  *    reuse it (you can't use two different loops on a sqlite3_stmt, of
-	  *    course). This happens with recursive uses of a cursor, for example */
+	/* There are three kinds of queries:
+	 * a) Cached queries: SELECT and UPDATE ones (cache_type)
+	 * b) Non-Cached queries: NONE ones (cache_type)
+	 * c) Forced Non-Cached: in case of a stmt being already in use, we can't
+	 *    reuse it (you can't use two different loops on a sqlite3_stmt, of
+	 *    course). This happens with recursive uses of a cursor, for example */
 
 	if (cache_type != TRACKER_DB_STATEMENT_CACHE_TYPE_NONE) {
 		stmt = g_hash_table_lookup (db_interface->dynamic_statements, full_query);
@@ -1029,6 +1097,13 @@ create_result_set_from_stmt (TrackerDBInterface  *interface,
 	columns = sqlite3_column_count (stmt);
 	result = SQLITE_OK;
 
+	/* Statement is going to start, check if we got a request to reset the
+	 * collator, and if so, do it. */
+	if (g_atomic_int_exchange_and_add (&interface->n_active_cursors, 1) == 0 &&
+	    g_atomic_int_compare_and_exchange (&(interface->collator_reset_requested), TRUE, FALSE)) {
+		tracker_db_interface_sqlite_reset_collator (interface);
+	}
+
 	while (result == SQLITE_OK  ||
 	       result == SQLITE_ROW) {
 
@@ -1059,7 +1134,17 @@ create_result_set_from_stmt (TrackerDBInterface  *interface,
 		}
 	}
 
-	if (result != SQLITE_DONE) {
+
+	if (result == SQLITE_DONE) {
+		/* Statement finished, check if we got a request to reset the
+		 * collator, and if so, do it.
+		 */
+		if (g_atomic_int_dec_and_test (&interface->n_active_cursors) &&
+		    g_atomic_int_compare_and_exchange (&(interface->collator_reset_requested), TRUE, FALSE)) {
+			tracker_db_interface_sqlite_reset_collator (interface);
+		}
+	} else {
+		g_atomic_int_add (&interface->n_active_cursors, -1);
 
 		/* This is rather fatal */
 		if (sqlite3_errcode (interface->db) == SQLITE_IOERR ||
@@ -1228,10 +1313,19 @@ static void
 tracker_db_cursor_finalize (GObject *object)
 {
 	TrackerDBCursor *cursor;
+	TrackerDBInterface *iface;
 	int i;
 
 	cursor = TRACKER_DB_CURSOR (object);
 
+	/* As soon as we finalize the cursor, check if we need a collator reset
+	 * and notify the iface about the removed cursor */
+	iface = cursor->ref_stmt->db_interface;
+	if (g_atomic_int_dec_and_test (&iface->n_active_cursors) &&
+	    g_atomic_int_compare_and_exchange (&(iface->collator_reset_requested), TRUE, FALSE)) {
+		tracker_db_interface_sqlite_reset_collator (iface);
+	}
+
 	g_free (cursor->types);
 
 	for (i = 0; i < cursor->n_variable_names; i++) {
@@ -1239,13 +1333,9 @@ tracker_db_cursor_finalize (GObject *object)
 	}
 	g_free (cursor->variable_names);
 
-	if (cursor->ref_stmt) {
-		cursor->ref_stmt->stmt_is_sunk = FALSE;
-		tracker_db_statement_sqlite_reset (cursor->ref_stmt);
-		g_object_unref (cursor->ref_stmt);
-	} else {
-		sqlite3_finalize (cursor->stmt);
-	}
+	cursor->ref_stmt->stmt_is_sunk = FALSE;
+	tracker_db_statement_sqlite_reset (cursor->ref_stmt);
+	g_object_unref (cursor->ref_stmt);
 
 	G_OBJECT_CLASS (tracker_db_cursor_parent_class)->finalize (object);
 }
@@ -1277,7 +1367,6 @@ async_ready_callback_wrapper (GObject      *source_object,
 	TrackerDBCursor *cursor = TRACKER_DB_CURSOR (source_object);
 	TrackerDBInterface *iface = cursor->ref_stmt->db_interface;
 
-	iface->pending = FALSE;
 	if (iface->outstanding_callback) {
 		GAsyncReadyCallback callback = iface->outstanding_callback;
 
@@ -1295,8 +1384,6 @@ tracker_db_cursor_iter_next_async (TrackerDBCursor     *cursor,
 {
 	GSimpleAsyncResult *res;
 
-	g_return_if_fail (!cursor->ref_stmt->db_interface->pending);
-	cursor->ref_stmt->db_interface->pending = TRUE;
 	cursor->ref_stmt->db_interface->outstanding_callback = callback;
 
 	res = g_simple_async_result_new (g_object_ref (cursor), async_ready_callback_wrapper, user_data, tracker_db_cursor_iter_next_async);
@@ -1342,21 +1429,29 @@ tracker_db_cursor_sqlite_new (sqlite3_stmt        *sqlite_stmt,
                               TrackerPropertyType *types,
                               gint                 n_types,
                               const gchar        **variable_names,
-                              gint                 n_variable_names)
+                              gint                 n_variable_names,
+                              gboolean             threadsafe)
 {
 	TrackerDBCursor *cursor;
+	TrackerDBInterface *iface;
+
+	/* As soon as we create a cursor, check if we need a collator reset
+	 * and notify the iface about the new cursor */
+	iface = ref_stmt->db_interface;
+	if (g_atomic_int_exchange_and_add (&iface->n_active_cursors, 1) == 0 &&
+	    g_atomic_int_compare_and_exchange (&(iface->collator_reset_requested), TRUE, FALSE)) {
+		tracker_db_interface_sqlite_reset_collator (iface);
+	}
 
 	cursor = g_object_new (TRACKER_TYPE_DB_CURSOR, NULL);
 
-	cursor->stmt = sqlite_stmt;
 	cursor->finished = FALSE;
 
-	if (ref_stmt) {
-		ref_stmt->stmt_is_sunk = TRUE;
-		cursor->ref_stmt = g_object_ref (ref_stmt);
-	} else {
-		cursor->ref_stmt = NULL;
-	}
+	cursor->threadsafe = threadsafe;
+
+	cursor->stmt = sqlite_stmt;
+	ref_stmt->stmt_is_sunk = TRUE;
+	cursor->ref_stmt = g_object_ref (ref_stmt);
 
 	if (types) {
 		gint i;
@@ -1432,7 +1527,6 @@ void
 tracker_db_cursor_rewind (TrackerDBCursor *cursor)
 {
 	g_return_if_fail (TRACKER_IS_DB_CURSOR (cursor));
-	g_return_if_fail (!cursor->ref_stmt->db_interface->pending);
 
 	sqlite3_reset (cursor->stmt);
 	cursor->finished = FALSE;
@@ -1443,8 +1537,6 @@ tracker_db_cursor_iter_next (TrackerDBCursor *cursor,
                              GCancellable    *cancellable,
                              GError         **error)
 {
-	g_return_val_if_fail (!cursor->ref_stmt->db_interface->pending, FALSE);
-
 	return db_cursor_iter_next (cursor, cancellable, error);
 }
 
@@ -1460,6 +1552,10 @@ db_cursor_iter_next (TrackerDBCursor *cursor,
 	if (!cursor->finished) {
 		guint result;
 
+		if (cursor->threadsafe) {
+			tracker_db_manager_lock ();
+		}
+
 		if (g_cancellable_is_cancelled (cancellable)) {
 			result = SQLITE_INTERRUPT;
 			sqlite3_reset (cursor->stmt);
@@ -1476,7 +1572,6 @@ db_cursor_iter_next (TrackerDBCursor *cursor,
 			             TRACKER_DB_INTERRUPTED,
 			             "Interrupted");
 		} else if (result != SQLITE_ROW && result != SQLITE_DONE) {
-
 			g_set_error (error,
 			             TRACKER_DB_INTERFACE_ERROR,
 			             TRACKER_DB_QUERY_ERROR,
@@ -1484,6 +1579,10 @@ db_cursor_iter_next (TrackerDBCursor *cursor,
 		}
 
 		cursor->finished = (result != SQLITE_ROW);
+
+		if (cursor->threadsafe) {
+			tracker_db_manager_unlock ();
+		}
 	}
 
 	return (!cursor->finished);
@@ -1619,7 +1718,7 @@ tracker_db_statement_start_cursor (TrackerDBStatement  *stmt,
 {
 	g_return_val_if_fail (!stmt->stmt_is_sunk, NULL);
 
-	return tracker_db_cursor_sqlite_new (stmt->stmt, stmt, NULL, 0, NULL, 0);
+	return tracker_db_cursor_sqlite_new (stmt->stmt, stmt, NULL, 0, NULL, 0, FALSE);
 }
 
 TrackerDBCursor *
@@ -1628,11 +1727,12 @@ tracker_db_statement_start_sparql_cursor (TrackerDBStatement   *stmt,
                                           gint                  n_types,
                                           const gchar         **variable_names,
                                           gint                  n_variable_names,
+                                          gboolean              threadsafe,
                                           GError              **error)
 {
 	g_return_val_if_fail (!stmt->stmt_is_sunk, NULL);
 
-	return tracker_db_cursor_sqlite_new (stmt->stmt, stmt, types, n_types, variable_names, n_variable_names);
+	return tracker_db_cursor_sqlite_new (stmt->stmt, stmt, types, n_types, variable_names, n_variable_names, threadsafe);
 }
 
 static void
