@@ -36,6 +36,7 @@
 #include <libtracker-common/tracker-date-time.h>
 #include <libtracker-common/tracker-file-utils.h>
 #include <libtracker-common/tracker-utils.h>
+#include <libtracker-common/tracker-locale.h>
 
 #if HAVE_TRACKER_FTS
 #include <libtracker-fts/tracker-fts.h>
@@ -158,6 +159,9 @@ static guint                 u_cache_size;
 
 static GStaticPrivate        interface_data_key = G_STATIC_PRIVATE_INIT;
 
+/* mutex used by singleton connection in libtracker-direct, not used by tracker-store */
+static GStaticMutex          global_mutex = G_STATIC_MUTEX_INIT;
+
 static const gchar *
 location_to_directory (TrackerDBLocation location)
 {
@@ -215,11 +219,11 @@ db_set_params (TrackerDBInterface *iface,
 	pragmas_file = g_getenv ("TRACKER_PRAGMAS_FILE");
 
 	if (pragmas_file && g_file_get_contents (pragmas_file, &queries, NULL, NULL)) {
-		g_debug ("PRAGMA's from file: %s", pragmas_file);
 		gchar *query = strtok (queries, "\n");
+		g_debug ("PRAGMA's from file: %s", pragmas_file);
 		while (query) {
 			g_debug ("  INIT query: %s", query);
-			tracker_db_interface_execute_query (iface, NULL, query);
+			tracker_db_interface_execute_query (iface, NULL, "%s", query);
 			query = strtok (NULL, "\n");
 		}
 		g_free (queries);
@@ -584,7 +588,7 @@ tracker_db_manager_locale_changed (void)
 	gboolean changed;
 
 	/* Get current collation locale */
-	current_locale = setlocale (LC_COLLATE, NULL);
+	current_locale = tracker_locale_get (TRACKER_LOCALE_COLLATE);
 
 	/* Get db locale */
 	db_locale = db_get_locale ();
@@ -602,20 +606,20 @@ tracker_db_manager_locale_changed (void)
 	}
 
 	g_free (db_locale);
+	g_free (current_locale);
 	return changed;
 }
 
 void
 tracker_db_manager_set_current_locale (void)
 {
-	const gchar *current_locale;
+	gchar *current_locale;
 
 	/* Get current collation locale */
-	current_locale = setlocale (LC_COLLATE, NULL);
-
+	current_locale = tracker_locale_get (TRACKER_LOCALE_COLLATE);
 	g_message ("Changing db locale to: '%s'", current_locale);
-
 	db_set_locale (current_locale);
+	g_free (current_locale);
 }
 
 static void
@@ -659,6 +663,7 @@ static void
 db_recreate_all (void)
 {
 	guint i;
+	gchar *locale;
 
 	/* We call an internal version of this function here
 	 * because at the time 'initialized' = FALSE and that
@@ -684,8 +689,10 @@ db_recreate_all (void)
 		dbs[i].iface = NULL;
 	}
 
+	locale = tracker_locale_get (TRACKER_LOCALE_COLLATE);
 	/* Initialize locale file */
-	db_set_locale (setlocale (LC_COLLATE, NULL));
+	db_set_locale (locale);
+	g_free (locale);
 }
 
 void
@@ -723,7 +730,8 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
                          guint                  select_cache_size,
                          guint                  update_cache_size,
                          TrackerBusyCallback    busy_callback,
-                         gpointer               busy_user_data)
+                         gpointer               busy_user_data,
+                         const gchar           *busy_operation)
 {
 	GType               etype;
 	TrackerDBVersion    version;
@@ -913,6 +921,7 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 				struct stat st;
 				GError *error = NULL;
 				TrackerDBStatement *stmt;
+				gchar *busy_status;
 
 				if (g_stat (dbs[i].abs_filename, &st) == 0) {
 					size = st.st_size;
@@ -931,10 +940,15 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 
 				loaded = TRUE;
 
+				/* Report OPERATION - STATUS */
+				busy_status = g_strdup_printf ("%s - %s",
+				                               busy_operation,
+				                               "Integrity checking");
 				tracker_db_interface_set_busy_handler (dbs[i].iface,
 				                                       busy_callback,
-				                                       "Integrity checking",
+				                                       busy_status,
 				                                       busy_user_data);
+				g_free (busy_status);
 
 				stmt = tracker_db_interface_create_statement (dbs[i].iface, TRACKER_DB_STATEMENT_CACHE_TYPE_NONE, &error,
 				                                              "PRAGMA integrity_check(1)");
@@ -1680,3 +1694,21 @@ tracker_db_manager_set_last_crawl_done (gboolean done)
 
 	g_free (filename);
 }
+
+void
+tracker_db_manager_lock (void)
+{
+	g_static_mutex_lock (&global_mutex);
+}
+
+gboolean
+tracker_db_manager_trylock (void)
+{
+	return g_static_mutex_trylock (&global_mutex);
+}
+
+void
+tracker_db_manager_unlock (void)
+{
+	g_static_mutex_unlock (&global_mutex);
+}
