@@ -562,7 +562,7 @@ tracker_xmp_read (const gchar    *buffer,
  * Returns: a newly allocated #TrackerXmpData struct if XMP data was
  * found, %NULL otherwise. Free the returned struct with tracker_xmp_free().
  *
- * Since: 0.9
+ * Since: 0.10
  **/
 TrackerXmpData *
 tracker_xmp_new (const gchar *buffer,
@@ -592,7 +592,7 @@ tracker_xmp_new (const gchar *buffer,
  * Frees @data and all #TrackerXmpData members. %NULL will produce a
  * a warning.
  *
- * Since: 0.9
+ * Since: 0.10
  **/
 void
 tracker_xmp_free (TrackerXmpData *data)
@@ -872,9 +872,11 @@ tracker_xmp_apply (TrackerSparqlBuilder *preupdate,
 	}
 
 	if (data->address || data->country || data->city) {
+		gchar *addruri;
+
 		tracker_sparql_builder_predicate (metadata, "mlo:location");
 
-		tracker_sparql_builder_object_blank_open (metadata);
+		tracker_sparql_builder_object_blank_open (metadata); /* GeoPoint */
 		tracker_sparql_builder_predicate (metadata, "a");
 		tracker_sparql_builder_object (metadata, "mlo:GeoPoint");
 
@@ -897,8 +899,43 @@ tracker_xmp_apply (TrackerSparqlBuilder *preupdate,
 			tracker_sparql_builder_predicate (metadata, "mlo:country");
 			tracker_sparql_builder_object_unvalidated (metadata, data->country);
 		}
-	
-		tracker_sparql_builder_object_blank_close (metadata);
+
+		addruri = tracker_sparql_get_uuid_urn ();
+
+		tracker_sparql_builder_predicate (metadata, "mlo:asPostalAddress");
+		tracker_sparql_builder_object_iri (metadata, addruri);
+
+		tracker_sparql_builder_object_blank_close (metadata); /* GeoPoint */
+
+		tracker_sparql_builder_insert_open (preupdate, NULL);
+		tracker_sparql_builder_subject_iri (preupdate, addruri);
+
+		g_free (addruri);
+
+		tracker_sparql_builder_predicate (preupdate, "a");
+		tracker_sparql_builder_object (preupdate, "nco:PostalAddress");
+
+		if (data->address) {
+			tracker_sparql_builder_predicate (preupdate, "nco:streetAddress");
+			tracker_sparql_builder_object_unvalidated (preupdate, data->address);
+		}
+
+		if (data->state) {
+			tracker_sparql_builder_predicate (preupdate, "nco:region");
+			tracker_sparql_builder_object_unvalidated (preupdate, data->state);
+		}
+
+		if (data->city) {
+			tracker_sparql_builder_predicate (preupdate, "nco:locality");
+			tracker_sparql_builder_object_unvalidated (preupdate, data->city);
+		}
+
+		if (data->country) {
+			tracker_sparql_builder_predicate (preupdate, "nco:country");
+			tracker_sparql_builder_object_unvalidated (preupdate, data->country);
+		}
+
+		tracker_sparql_builder_insert_close (preupdate);
 	}
 
 	return TRUE;
