@@ -496,7 +496,8 @@ extract_metadata (MetadataExtractor      *extractor,
                   gchar                 **scount)
 {
 	const gchar *temp;
-	gchar *s, *make = NULL, *model = NULL, *manuf = NULL;
+	gchar *make = NULL, *model = NULL, *manuf = NULL;
+	gchar *composer = NULL, *albumname = NULL, *genre = NULL;
 	gboolean ret;
 	gint count;
 	gboolean needs_audio = FALSE;
@@ -509,6 +510,7 @@ extract_metadata (MetadataExtractor      *extractor,
 		gchar *performer_uri = NULL;
 		gchar *composer_uri = NULL;
 		gchar *album_uri = NULL;
+		gchar *album_disc_uri = NULL;
 
 		if (extractor->mime == EXTRACT_MIME_GUESS) {
 			gchar *video_codec = NULL, *audio_codec = NULL;
@@ -565,43 +567,36 @@ extract_metadata (MetadataExtractor      *extractor,
 			g_free (performer);
 			g_free (artist_local);
 
-			s = NULL;
-			gst_tag_list_get_string (extractor->tagcache, GST_TAG_COMPOSER, &s);
-
-			if (s) {
-				composer_uri = tracker_sparql_escape_uri_printf ("urn:artist:%s", s);
+			gst_tag_list_get_string (extractor->tagcache, GST_TAG_COMPOSER, &composer);
 
+			if (composer) {
+				composer_uri = tracker_sparql_escape_uri_printf ("urn:artist:%s", composer);
 				tracker_sparql_builder_insert_open (preupdate, NULL);
-
 				tracker_sparql_builder_subject_iri (preupdate, composer_uri);
 				tracker_sparql_builder_predicate (preupdate, "a");
 				tracker_sparql_builder_object (preupdate, "nmm:Artist");
-
 				tracker_sparql_builder_predicate (preupdate, "nmm:artistName");
-				tracker_sparql_builder_object_unvalidated (preupdate, s);
-
+				tracker_sparql_builder_object_unvalidated (preupdate, composer);
 				tracker_sparql_builder_insert_close (preupdate);
-
-				g_free (s);
+				g_free (composer);
 			}
 
 		}
 
 		/* Audio */
-		s = NULL;
 
 		if (extractor->mime == EXTRACT_MIME_AUDIO) {
-			gst_tag_list_get_string (extractor->tagcache, GST_TAG_ALBUM, &s);
+			gst_tag_list_get_string (extractor->tagcache, GST_TAG_ALBUM, &albumname);
 		}
 
-		if (s) {
+		if (albumname) {
 			gboolean has_it;
 			guint count;
 			gdouble gain;
 
 			needs_audio = TRUE;
 
-			album_uri = tracker_sparql_escape_uri_printf ("urn:album:%s", s);
+			album_uri = tracker_sparql_escape_uri_printf ("urn:album:%s", albumname);
 
 			tracker_sparql_builder_insert_open (preupdate, NULL);
 
@@ -612,7 +607,7 @@ extract_metadata (MetadataExtractor      *extractor,
 			 * tracker_sparql_builder_predicate (preupdate, "nie:title");
 			 */
 			tracker_sparql_builder_predicate (preupdate, "nmm:albumTitle");
-			tracker_sparql_builder_object_unvalidated (preupdate, s);
+			tracker_sparql_builder_object_unvalidated (preupdate, albumname);
 
 			if (artist_uri) {
 				tracker_sparql_builder_predicate (preupdate, "nmm:albumArtist");
@@ -631,7 +626,6 @@ extract_metadata (MetadataExtractor      *extractor,
 				tracker_sparql_builder_predicate (preupdate, "nmm:albumTrackCount");
 				tracker_sparql_builder_object_variable (preupdate, "unknown");
 				tracker_sparql_builder_delete_close (preupdate);
-
 				tracker_sparql_builder_where_open (preupdate);
 				tracker_sparql_builder_subject_iri (preupdate, album_uri);
 				tracker_sparql_builder_predicate (preupdate, "nmm:albumTrackCount");
@@ -639,11 +633,9 @@ extract_metadata (MetadataExtractor      *extractor,
 				tracker_sparql_builder_where_close (preupdate);
 
 				tracker_sparql_builder_insert_open (preupdate, NULL);
-
 				tracker_sparql_builder_subject_iri (preupdate, album_uri);
 				tracker_sparql_builder_predicate (preupdate, "nmm:albumTrackCount");
 				tracker_sparql_builder_object_int64 (preupdate, count);
-
 				tracker_sparql_builder_insert_close (preupdate);
 			}
 
@@ -651,27 +643,41 @@ extract_metadata (MetadataExtractor      *extractor,
 			                                GST_TAG_ALBUM_VOLUME_NUMBER,
 			                                &count);
 
-			if (has_it) {
-				tracker_sparql_builder_delete_open (preupdate, NULL);
-				tracker_sparql_builder_subject_iri (preupdate, album_uri);
-				tracker_sparql_builder_predicate (preupdate, "nmm:setNumber");
-				tracker_sparql_builder_object_variable (preupdate, "unknown");
-				tracker_sparql_builder_delete_close (preupdate);
+			album_disc_uri = tracker_sparql_escape_uri_printf ("urn:album-disc:%s:Disc%d",
+			                                                   albumname,
+			                                                   has_it ? count : 1);
 
-				tracker_sparql_builder_where_open (preupdate);
-				tracker_sparql_builder_subject_iri (preupdate, album_uri);
-				tracker_sparql_builder_predicate (preupdate, "nmm:setNumber");
-				tracker_sparql_builder_object_variable (preupdate, "unknown");
-				tracker_sparql_builder_where_close (preupdate);
+			tracker_sparql_builder_delete_open (preupdate, NULL);
+			tracker_sparql_builder_subject_iri (preupdate, album_disc_uri);
+			tracker_sparql_builder_predicate (preupdate, "nmm:setNumber");
+			tracker_sparql_builder_object_variable (preupdate, "unknown");
+			tracker_sparql_builder_delete_close (preupdate);
+			tracker_sparql_builder_where_open (preupdate);
+			tracker_sparql_builder_subject_iri (preupdate, album_disc_uri);
+			tracker_sparql_builder_predicate (preupdate, "nmm:setNumber");
+			tracker_sparql_builder_object_variable (preupdate, "unknown");
+			tracker_sparql_builder_where_close (preupdate);
 
-				tracker_sparql_builder_insert_open (preupdate, NULL);
+			tracker_sparql_builder_delete_open (preupdate, NULL);
+			tracker_sparql_builder_subject_iri (preupdate, album_disc_uri);
+			tracker_sparql_builder_predicate (preupdate, "nmm:albumDiscAlbum");
+			tracker_sparql_builder_object_variable (preupdate, "unknown");
+			tracker_sparql_builder_delete_close (preupdate);
+			tracker_sparql_builder_where_open (preupdate);
+			tracker_sparql_builder_subject_iri (preupdate, album_disc_uri);
+			tracker_sparql_builder_predicate (preupdate, "nmm:albumDiscAlbum");
+			tracker_sparql_builder_object_variable (preupdate, "unknown");
+			tracker_sparql_builder_where_close (preupdate);
 
-				tracker_sparql_builder_subject_iri (preupdate, album_uri);
-				tracker_sparql_builder_predicate (preupdate, "nmm:setNumber");
-				tracker_sparql_builder_object_int64 (preupdate, count);
-
-				tracker_sparql_builder_insert_close (preupdate);
-			}
+			tracker_sparql_builder_insert_open (preupdate, NULL);
+			tracker_sparql_builder_subject_iri (preupdate, album_disc_uri);
+			tracker_sparql_builder_predicate (preupdate, "a");
+			tracker_sparql_builder_object (preupdate, "nmm:MusicAlbumDisc");
+			tracker_sparql_builder_predicate (preupdate, "nmm:setNumber");
+			tracker_sparql_builder_object_int64 (preupdate, has_it ? count : 1);
+			tracker_sparql_builder_predicate (preupdate, "nmm:albumDiscAlbum");
+			tracker_sparql_builder_object_iri (preupdate, album_uri);
+			tracker_sparql_builder_insert_close (preupdate);
 
 			has_it = gst_tag_list_get_double (extractor->tagcache,
 			                                  GST_TAG_ALBUM_GAIN,
@@ -725,7 +731,7 @@ extract_metadata (MetadataExtractor      *extractor,
 				tracker_sparql_builder_insert_close (preupdate);
 			}
 
-			*album = s;
+			*album = albumname;
 
 		}
 
@@ -746,13 +752,12 @@ extract_metadata (MetadataExtractor      *extractor,
 			tracker_sparql_builder_object (metadata, "nmm:Photo");
 		}
 
-		s = NULL;
-		gst_tag_list_get_string (extractor->tagcache, GST_TAG_GENRE, &s);
-		if (s && strcmp (s, "Unknown") != 0) {
+		gst_tag_list_get_string (extractor->tagcache, GST_TAG_GENRE, &genre);
+		if (g_strcmp0 (genre, "Unknown") != 0) {
 			tracker_sparql_builder_predicate (metadata, "nfo:genre");
-			tracker_sparql_builder_object_unvalidated (metadata, s);
+			tracker_sparql_builder_object_unvalidated (metadata, genre);
 		}
-		g_free (s);
+		g_free (genre);
 
 		add_string_gst_tag (metadata, uri, "nie:title", extractor->tagcache, GST_TAG_TITLE);
 		add_string_gst_tag (metadata, uri, "nie:copyright", extractor->tagcache, GST_TAG_COPYRIGHT);
@@ -843,11 +848,17 @@ extract_metadata (MetadataExtractor      *extractor,
 				tracker_sparql_builder_predicate (metadata, "nmm:musicAlbum");
 				tracker_sparql_builder_object_iri (metadata, album_uri);
 			}
+
+			if (album_disc_uri) {
+				tracker_sparql_builder_predicate (metadata, "nmm:musicAlbumDisc");
+				tracker_sparql_builder_object_iri (metadata, album_disc_uri);
+			}
 		}
 
 		g_free (performer_uri);
 		g_free (composer_uri);
 		g_free (album_uri);
+		g_free (album_disc_uri);
 
 		add_string_gst_tag (metadata, uri, "nfo:codec", extractor->tagcache, GST_TAG_AUDIO_CODEC);
 	} else {
